@@ -18,31 +18,29 @@
  */
 package org.apache.flume.channel.file;
 
-import java.io.DataInputStream;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.IOException;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.List;
-
+import com.google.common.collect.Lists;
+import com.google.common.io.Files;
+import com.google.protobuf.InvalidProtocolBufferException;
 import junit.framework.Assert;
-
 import org.apache.commons.io.FileUtils;
+import org.apache.flume.channel.file.proto.ProtosFactory;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
 
-import com.google.common.collect.Lists;
-import com.google.common.io.Files;
-import com.google.protobuf.InvalidProtocolBufferException;
+import java.io.DataInputStream;
+import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileOutputStream;
+import java.io.IOException;
 import java.io.RandomAccessFile;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.List;
 import java.util.Random;
-import org.apache.flume.channel.file.proto.ProtosFactory;
 
 public class TestEventQueueBackingStoreFactory {
-  static final List<Long> pointersInTestCheckpoint = Arrays.asList(new Long[] {
+  static final List<Long> pointersInTestCheckpoint = Arrays.asList(new Long[]{
       8589936804L,
       4294969563L,
       12884904153L,
@@ -59,6 +57,7 @@ public class TestEventQueueBackingStoreFactory {
   File inflightTakes;
   File inflightPuts;
   File queueSetDir;
+
   @Before
   public void setup() throws IOException {
     baseDir = Files.createTempDir();
@@ -67,42 +66,46 @@ public class TestEventQueueBackingStoreFactory {
     inflightPuts = new File(baseDir, "puts");
     queueSetDir = new File(baseDir, "queueset");
     TestUtils.copyDecompressed("fileformat-v2-checkpoint.gz", checkpoint);
-
   }
+
   @After
   public void teardown() {
     FileUtils.deleteQuietly(baseDir);
   }
+
   @Test
   public void testWithNoFlag() throws Exception {
     verify(EventQueueBackingStoreFactory.get(checkpoint, 10, "test"),
-        Serialization.VERSION_3, pointersInTestCheckpoint);
+           Serialization.VERSION_3, pointersInTestCheckpoint);
   }
+
   @Test
   public void testWithFlag() throws Exception {
     verify(EventQueueBackingStoreFactory.get(checkpoint, 10, "test", true),
-        Serialization.VERSION_3, pointersInTestCheckpoint);
+           Serialization.VERSION_3, pointersInTestCheckpoint);
   }
+
   @Test
   public void testNoUprade() throws Exception {
     verify(EventQueueBackingStoreFactory.get(checkpoint, 10, "test", false),
-        Serialization.VERSION_2, pointersInTestCheckpoint);
+           Serialization.VERSION_2, pointersInTestCheckpoint);
   }
-  @Test (expected = BadCheckpointException.class)
+
+  @Test(expected = BadCheckpointException.class)
   public void testDecreaseCapacity() throws Exception {
     Assert.assertTrue(checkpoint.delete());
-    EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-            get(checkpoint, 10, "test");
+    EventQueueBackingStore backingStore =
+        EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
     backingStore.close();
     EventQueueBackingStoreFactory.get(checkpoint, 9, "test");
     Assert.fail();
   }
 
-  @Test (expected = BadCheckpointException.class)
+  @Test(expected = BadCheckpointException.class)
   public void testIncreaseCapacity() throws Exception {
     Assert.assertTrue(checkpoint.delete());
-    EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-            get(checkpoint, 10, "test");
+    EventQueueBackingStore backingStore =
+        EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
     backingStore.close();
     EventQueueBackingStoreFactory.get(checkpoint, 11, "test");
     Assert.fail();
@@ -112,22 +115,21 @@ public class TestEventQueueBackingStoreFactory {
   public void testNewCheckpoint() throws Exception {
     Assert.assertTrue(checkpoint.delete());
     verify(EventQueueBackingStoreFactory.get(checkpoint, 10, "test", false),
-        Serialization.VERSION_3, Collections.<Long>emptyList());
+           Serialization.VERSION_3, Collections.<Long>emptyList());
   }
 
-  @Test (expected = BadCheckpointException.class)
+  @Test(expected = BadCheckpointException.class)
   public void testCheckpointBadVersion() throws Exception {
-     RandomAccessFile writer = new RandomAccessFile(checkpoint, "rw");
+    RandomAccessFile writer = new RandomAccessFile(checkpoint, "rw");
     try {
-    EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-            get(checkpoint, 10, "test");
-    backingStore.close();
-    writer.seek(
-            EventQueueBackingStoreFile.INDEX_VERSION * Serialization.SIZE_OF_LONG);
-    writer.writeLong(94L);
-    writer.getFD().sync();
+      EventQueueBackingStore backingStore =
+          EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
+      backingStore.close();
+      writer.seek(EventQueueBackingStoreFile.INDEX_VERSION * Serialization.SIZE_OF_LONG);
+      writer.writeLong(94L);
+      writer.getFD().sync();
 
-    backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
+      backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
     } finally {
       writer.close();
     }
@@ -138,15 +140,13 @@ public class TestEventQueueBackingStoreFactory {
     RandomAccessFile writer = new RandomAccessFile(checkpoint, "rw");
 
     try {
-    EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-            get(checkpoint, 10, "test");
-    backingStore.close();
-    writer.seek(
-            EventQueueBackingStoreFile.INDEX_CHECKPOINT_MARKER *
-            Serialization.SIZE_OF_LONG);
-    writer.writeLong(EventQueueBackingStoreFile.CHECKPOINT_INCOMPLETE);
-    writer.getFD().sync();
-    backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
+      EventQueueBackingStore backingStore =
+          EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
+      backingStore.close();
+      writer.seek(EventQueueBackingStoreFile.INDEX_CHECKPOINT_MARKER * Serialization.SIZE_OF_LONG);
+      writer.writeLong(EventQueueBackingStoreFile.CHECKPOINT_INCOMPLETE);
+      writer.getFD().sync();
+      backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
     } finally {
       writer.close();
     }
@@ -156,12 +156,10 @@ public class TestEventQueueBackingStoreFactory {
   public void testCheckpointVersionNotEqualToMeta() throws Exception {
     RandomAccessFile writer = new RandomAccessFile(checkpoint, "rw");
     try {
-      EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-              get(checkpoint, 10, "test");
+      EventQueueBackingStore backingStore =
+          EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
       backingStore.close();
-      writer.seek(
-              EventQueueBackingStoreFile.INDEX_VERSION
-              * Serialization.SIZE_OF_LONG);
+      writer.seek(EventQueueBackingStoreFile.INDEX_VERSION * Serialization.SIZE_OF_LONG);
       writer.writeLong(2L);
       writer.getFD().sync();
       backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
@@ -174,8 +172,8 @@ public class TestEventQueueBackingStoreFactory {
   public void testCheckpointVersionNotEqualToMeta2() throws Exception {
     FileOutputStream os = null;
     try {
-      EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-              get(checkpoint, 10, "test");
+      EventQueueBackingStore backingStore =
+          EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
       backingStore.close();
       Assert.assertTrue(checkpoint.exists());
       Assert.assertTrue(Serialization.getMetaDataFile(checkpoint).length() != 0);
@@ -183,8 +181,7 @@ public class TestEventQueueBackingStoreFactory {
       ProtosFactory.Checkpoint meta = ProtosFactory.Checkpoint.parseDelimitedFrom(is);
       Assert.assertNotNull(meta);
       is.close();
-      os = new FileOutputStream(
-              Serialization.getMetaDataFile(checkpoint));
+      os = new FileOutputStream(Serialization.getMetaDataFile(checkpoint));
       meta.toBuilder().setVersion(2).build().writeDelimitedTo(os);
       os.flush();
       backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
@@ -197,12 +194,10 @@ public class TestEventQueueBackingStoreFactory {
   public void testCheckpointOrderIdNotEqualToMeta() throws Exception {
     RandomAccessFile writer = new RandomAccessFile(checkpoint, "rw");
     try {
-      EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-              get(checkpoint, 10, "test");
+      EventQueueBackingStore backingStore =
+          EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
       backingStore.close();
-      writer.seek(
-              EventQueueBackingStoreFile.INDEX_WRITE_ORDER_ID
-              * Serialization.SIZE_OF_LONG);
+      writer.seek(EventQueueBackingStoreFile.INDEX_WRITE_ORDER_ID * Serialization.SIZE_OF_LONG);
       writer.writeLong(2L);
       writer.getFD().sync();
       backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
@@ -215,8 +210,8 @@ public class TestEventQueueBackingStoreFactory {
   public void testCheckpointOrderIdNotEqualToMeta2() throws Exception {
     FileOutputStream os = null;
     try {
-      EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-              get(checkpoint, 10, "test");
+      EventQueueBackingStore backingStore =
+          EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
       backingStore.close();
       Assert.assertTrue(checkpoint.exists());
       Assert.assertTrue(Serialization.getMetaDataFile(checkpoint).length() != 0);
@@ -225,7 +220,7 @@ public class TestEventQueueBackingStoreFactory {
       Assert.assertNotNull(meta);
       is.close();
       os = new FileOutputStream(
-              Serialization.getMetaDataFile(checkpoint));
+          Serialization.getMetaDataFile(checkpoint));
       meta.toBuilder().setWriteOrderID(1).build().writeDelimitedTo(os);
       os.flush();
       backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
@@ -234,11 +229,10 @@ public class TestEventQueueBackingStoreFactory {
     }
   }
 
-
   @Test(expected = BadCheckpointException.class)
   public void testTruncateMeta() throws Exception {
-    EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-            get(checkpoint, 10, "test");
+    EventQueueBackingStore backingStore =
+        EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
     backingStore.close();
     Assert.assertTrue(checkpoint.exists());
     File metaFile = Serialization.getMetaDataFile(checkpoint);
@@ -250,10 +244,10 @@ public class TestEventQueueBackingStoreFactory {
     backingStore = EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
   }
 
-  @Test (expected = InvalidProtocolBufferException.class)
+  @Test(expected = InvalidProtocolBufferException.class)
   public void testCorruptMeta() throws Throwable {
-    EventQueueBackingStore backingStore = EventQueueBackingStoreFactory.
-            get(checkpoint, 10, "test");
+    EventQueueBackingStore backingStore =
+        EventQueueBackingStoreFactory.get(checkpoint, 10, "test");
     backingStore.close();
     Assert.assertTrue(checkpoint.exists());
     File metaFile = Serialization.getMetaDataFile(checkpoint);
@@ -270,17 +264,13 @@ public class TestEventQueueBackingStoreFactory {
     }
   }
 
-
-
-
   private void verify(EventQueueBackingStore backingStore, long expectedVersion,
-      List<Long> expectedPointers)
-      throws Exception {
-    FlumeEventQueue queue = new FlumeEventQueue(backingStore, inflightTakes,
-        inflightPuts, queueSetDir);
+                      List<Long> expectedPointers) throws Exception {
+    FlumeEventQueue queue =
+        new FlumeEventQueue(backingStore, inflightTakes, inflightPuts, queueSetDir);
     List<Long> actualPointers = Lists.newArrayList();
     FlumeEventPointer ptr;
-    while((ptr = queue.removeHead(0L)) != null) {
+    while ((ptr = queue.removeHead(0L)) != null) {
       actualPointers.add(ptr.toLong());
     }
     Assert.assertEquals(expectedPointers, actualPointers);
